package com.seafile.seadroid2.ui;

import java.util.ArrayList;

import com.seafile.seadroid2.BrowserActivity;
import com.seafile.seadroid2.R;
import com.seafile.seadroid2.Utils;
import com.seafile.seadroid2.data.SeafDirent;
import com.seafile.seadroid2.data.SeafItem;
import com.seafile.seadroid2.data.SeafStarredFile;

import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.ImageView;
import android.widget.TextView;

public class StarredItemAdapter extends BaseAdapter {

    private ArrayList<SeafItem> items;
    private BrowserActivity mActivity;

    public StarredItemAdapter(BrowserActivity activity) {
        this.mActivity = activity;
        items = new ArrayList<SeafItem>();
    }
    
    @Override
    public int getCount() {
        return items.size();
    }

    public void clear() {
        items.clear();
    }
    
    public void add(SeafItem entry) {
        items.add(entry);
    }
    
    public void notifyChanged() {
        notifyDataSetChanged();
    }
    
    @Override
    public SeafItem getItem(int position) {
        return items.get(position);
    }

    @Override
    public long getItemId(int position) {
        return position;
    }

    @Override
    public View getView(int position, View convertView, ViewGroup parent) {
        SeafItem item = items.get(position);
        View view = convertView;
        Viewholder viewHolder;

        if (convertView == null) {
            view = LayoutInflater.from(mActivity).inflate(R.layout.list_item_entry, null);
            TextView title = (TextView) view.findViewById(R.id.list_item_title);
            TextView subtitle = (TextView) view.findViewById(R.id.list_item_subtitle);
            ImageView icon = (ImageView) view.findViewById(R.id.list_item_icon);
            ImageView action = (ImageView) view.findViewById(R.id.list_item_action);
            viewHolder = new Viewholder(title, subtitle, icon, action);
            view.setTag(viewHolder);
        } else {
            viewHolder = (Viewholder) convertView.getTag();
        }

        int iconID = ((SeafStarredFile)item).getIcon();
        viewHolder.icon.setImageResource(iconID);
        viewHolder.title.setText(((SeafStarredFile)item).getTitle());
        viewHolder.subtitle.setText(((SeafStarredFile)item).getSubtitle());

        //setFileView((SeafDirent)item, viewHolder, position);

        return view;
    }
    
    
    private class Viewholder {
        TextView title, subtitle;
        ImageView icon, action;

        public Viewholder(TextView title, TextView subtitle, ImageView icon, ImageView action) {
            super();
            this.icon = icon;
            this.action = action;
            this.title = title;
            this.subtitle = subtitle;
        }
    }
    

}
